<?php

//
// innovaphone incoming fax call in PHP script
// copyright (c) innovaphone 2012 - 2015
//

$path=dirname(__FILE__);
require_once("$path/../include.php");
require_once("classes/translator.class.php");
require_once("classes/inputpage.class.php");
require_once("classes/mail/innovaphone.mailbox.class.php");
require_once("$path/../app-classes/innovaphone.fax.config.class.php");
require_once("$path/../app-classes/innovaphone.fax.class.php");
require_once("$path/../app-classes/innovaphone.fax.database.class.php");
require_once("$path/../app-classes/innovaphone.fax.shm.class.php");
require_once("$path/../app-classes/innovaphone.pbx.class.php");
require_once("$path/../app-classes/innovaphone.pbx.info.class.php");

$version = file_get_contents("$path/../version");

declare(ticks = 1);
?>

innovaphone incoming fax call in PHP script
copyright (c) innovaphone 2012 - 2014
<?php

print "Version " . $version . "\n\n";

$help = false;
foreach($argv as $arg) {
    if(in_array($arg, array('--help', '-help', '-h', '-?'))) {
        $help = true;
    }
}

if(!is_array($_GET) && ($argc < 3 || $help)) {
?>
    Usage:
  <?php echo $argv[0]; ?> <id> <confId>

<id> is the configuration database id specific for this server instance.
<confId> is the conference id of the call which should be handled.

With the --help, -help, -h, or -? options, you can get this help.

<?php
exit(1);
}

$configurationId        = null;
$confId                 = null;

if($argc >= 2) {
    $configurationId    = intval($argv[1]);
    $confId             = $argv[2];
}
else if(is_array($_GET)) {
    $configurationId    = intval($_GET['id']);
    $confId             = $_GET['confId'];
}


setlocale(LC_CTYPE, "en_US.UTF-8");

function getLogDate() {
    $now = new DateTime();
    return date('Ymd-His') . "  -  ";
}

function sig_handler($nsig)
{
    print "\n";
    print getLogDate() . "terminated($nsig).\n";
    exit;
}

pcntl_signal(SIGTERM, "sig_handler", false);
pcntl_signal(SIGINT, "sig_handler", false);
pcntl_signal(SIGHUP, "sig_handler", false);


function createGuid() {
    $time   = explode(" ", microtime());
    $exec       = "/sbin/ifconfig eth0 | grep 'inet addr:' | cut -d: -f2 | awk '{ print $1}'";
    $outputs    = array();
    $ret        = false;
    exec($exec, $outputs, $ret);
    $ip     = explode('.', is_array($outputs) ? $outputs[0] : "");
    $hex    = "";
    for($i = 0; $i <= count($ip) - 1; $i++) {
        $hex .= substr("0" . dechex($ip[$i]), -2);
    }
    
    return sprintf('%08s-%04s-%04x-%08s-%04x',
        substr("00000000" . dechex($time[1]), -8),             // get 8HEX of unixtime
        substr("0000" . dechex(round($time[0] * 65536)), -4),  // get 4HEX of microtime
        mt_rand(0, 0xffff),
        $hex,
        mt_rand(0, 0xffff));
}

function terminateSession(&$pbx) {
    try {
        $p = $pbx->End($pbx->getSession());
    }
    catch(Exception $e) {
        return false;
    }
    return true;
}

function shutdown_function() {
    global $pbx;
    global $faxShm;
    global $jobGuid;
    if($faxShm && $jobGuid) {
        $faxShm->unlockJob($jobGuid);
    }
    if($pbx && $pbx->getKey()) {
        terminateSession($pbx);
    }
    print "\n";
}

register_shutdown_function('shutdown_function');

$pbx            = null;
$faxShm         = null;
$jobGuid        = null;


function handleError(&$faxDatabase, &$faxJob, &$cfg, $info) {
    $error      = "";
    $type       = "";

    if(array_key_exists('<type>', $info)) {
        $type = $info['<type>'];
    }                    
    if($type == 'NoEmailAddress') {
        $error = "Email address not configured";
    }
    else if($type == 'RightsNoUser') {
        $error = "User not found";
    }
    else if($type == 'RightsNoLicense') {
        $error = "No user license";
    }
    else if($type == 'RightsCheckFailed') {
        $error = "License check failed.";
    }
    else {
        $error = $type;
    }
    
    if($faxDatabase && $faxJob) {
        print getLogDate() . "Updating database... ";
        $faxJob->setProgress('closed');
        $faxJob->setResult('error');
        $faxJob->setErrorCause('by user');
        $faxJob->setErrorDescription($error);
        $faxJob = $faxDatabase->updateFaxJob($faxJob);
        if(!$faxJob) {
            print "failed.\n";
        }
        else {
            print "done.\n";
        }
    }

    print getLogDate() . "Connecting to mailbox... ";
    $mailbox    = new InnovaphoneMailbox($cfg->mailHost, $cfg->mailPort, $cfg->mailUser, $cfg->mailPassword, $cfg->mailFolder);
    if(!$mailbox->isConnected()) {
        print "failed.\n";
    }
    else {
        print "done.\n";

        $to         = $cfg->mailAdmin;
        if($faxJob && $faxJob->getUserH323() != "") {
            $to     = $faxJob->getUserH323();
        }
        $from       = imap_rfc822_write_address('faxserver', $cfg->faxServerDomain, $cfg->name);
        
        $language   = array_key_exists('<language>', $info) ? $info['<language>'] : $cfg->lang;
        $subject    = $cfg->getString(FaxServerStrings::msgSubjectError, $language);
        $message    = $cfg->getString(FaxServerStrings::msgError, $language);
            
        if($type != "") {
            $message_type   = "FaxServerStrings::msgError" . $type;
            $message1       = defined($message_type) ? $cfg->getString(constant($message_type), $language) : "";
            if($message1 != "") {
                $message    = $message1;
            }
            $subject_type   = "FaxServerStrings::msgSubjectError" . $type;
            $subject1       = defined($subject_type) ? $cfg->getString(constant($subject_type), $language) : "";
            if($subject1 != "") {
                $subject    = $subject1;
            }
        }

        $message = strtr($message, $info);
        $subject = strtr($subject, $info);
    
        print getLogDate() . "Sending status... ";
        if(!$mailbox->sendMail($to, $from, $subject, $message)) {
            print "failed.\n";
        }
        else {
            print "done.\n";
        }
    }

    die($error . "\n");
}


print getLogDate() . "Reading configuration... ";
$cfg     = new FaxServer();
$cfg->read("", $configurationId);
if(!$cfg) {
    print "failed.\n";
    die("Could not read configuration.");
}
print "done.\n";


print getLogDate() . "Attaching to shared memory... ";
$faxShm     = new InnovaphoneFaxShm($cfg->shmId);
if(!$faxShm->isAttached()) {
    print "failed.\n";
    die("Could not attach to the SHM.");
}
print "done.\n";


print getLogDate() . "Connecting to database... ";
$faxDatabase    = new InnovaphoneFaxDatabase();
if(!$faxDatabase->isConnected()) {
    print "failed.\n";
    die("Could not connect to the database.");
}
$faxDatabase->setServerId($cfg->id);
print "done.\n";


do {
    $jobGuid    = createGuid();    
    $faxJob     = $faxDatabase->getFaxJob($jobGuid);
} while($faxJob);


print getLogDate() . "Trying to lock job... ";
$ret = $faxShm->lockJob($jobGuid, $confId, null);
if(!$ret) {
    $faxShm = 0;
    print "failed.\n";
    die("Could not lock fax job.");
}
print "done.\n";


$faxCall = $faxShm->findCall($confId);
if(!$faxCall) {
    die("Could not find fax call with conf id $confId in the SHM.");
}
if($faxCall->getWebdavFileName() != "") {
    die("Webdav file name is set in the SHM call data.");
}


$server     = $cfg->soapHost;
$port       = $cfg->soapPort;
$https      = $cfg->https;
$httpu      = $cfg->soapUser;
$httpp      = $cfg->soapPassword;
$userName   = $cfg->soapFaxUser;
$dynPbxId   = $cfg->soapDynPbx;
$webdavUrl  = $cfg->webdavURL;

print getLogDate() . "Connecting to PBX " . $server . "... ";
$pbx = new InnovaphonePBX(
    $server,
    $port,
    $https,
    $httpu,
    $httpp,
    $userName,
    "innovaphone PHP fax call in [" . $version . "]",
    $cfg->trace ? array("trace" => 1) : null,
    dirname(__FILE__) . '/../wsdl/pbx10_00.wsdl',
    $dynPbxId);
    
if($pbx->getKey() == 0) {
    print "failed.\n";
    die("\nFailed to login to PBX: " . $pbx->getFaultString() . "\n");
}
print "connected.\n\n";

$pbx->showVersion();
print "\n";


try {
    $userHandle = $pbx->UserInitialize($pbx->getSession(), $userName, true, false, "");
    if(!$userHandle) {
        die("Could not subscribe to user {$userName}.\n");
    }
}
catch(Exception $e) {
    die("Connection failure.\n");
}

$fax = new InnovaphoneFax($pbx, $userName);
$pbxInfo = &$fax;

if(!$pbxInfo->initialize()) {
    die("Could not initialize the PBX data.\n");
}

$user = $pbxInfo->getUser($userName);
if(!$user) {
    die("User is not in poll list.\n");
}

$pbxInfo->purge();


print getLogDate() . "Searching for the call... ";
$call = $fax->findCall($confId);
if(!$call) {
    print "failed.\n";
    die("No call found.");
}
$callHandle = $call->getCallHandle();
print "found.\n";


print getLogDate() . "Creating database entry... ";
$faxJob = $faxDatabase->addFaxJob(new InnovaphoneFaxJob($jobGuid, 'incoming'));
if(!$faxJob) {
    print "failed.\n";
    die("Could not add database object.");
}
print "done.\n";


$faxJob->setRemoteNumber($call->getCallingNumber());
$incomplete             = true;
$destinationUser        = 0;
$destinationUserNumber  = "";
$calledNumber           = $call->getCalledNumber();

if($call->getCalledNumber() == $user->getNumber()) {
    $destinationUser = $user;
    $incomplete = false;
}
else {
    $destinationUser        = $fax->findDestinationUser($call->getUserHandle(), $call->getCalledNumber(), $cfg->faxUserNumberPrefix, $incomplete, true);
}
if(!$incomplete && $destinationUser) {
    if($destinationUser != $user) {
        $destinationUserNumber = $destinationUser->getSendNumber() != "" ? $destinationUser->getSendNumber() : $destinationUser->getNumber();
        if($destinationUser->getPseudoType() == "map") {
            $calledNumber = $destinationUser->getNumber();
        }
    }
    $faxJob->setUserName($destinationUser->getName());
    $faxJob->setUserH323($destinationUser->getEmail() ? $destinationUser->getEmail() : "");
    $faxJob->setUserNumber($calledNumber);
    if($cfg->getLocationLanguage($destinationUser->getLocation())) {
        $faxJob->setUserLanguage($cfg->getLocationLanguage($destinationUser->getLocation()));
    }
}

$faxJob = $faxDatabase->updateFaxJob($faxJob);
if(!$faxJob) {
    die("Could not update database object.");
}

if(!$destinationUser->getEmail()) {
    print getLogDate() . "No email address configured for the user.\n";

    $fax->handleIncomingCall($callHandle, "", "", "", true);

    $info = array();
    $info['<type>']             = 'NoEmailAddress';
    $info['<sender>']           = $faxJob->getUserName();
    $info['<user-name>']        = $faxJob->getUserName();
    $info['<remote-number>']    = $faxJob->getRemoteNumber();

    handleError($faxDatabase, $faxJob, $cfg, $info);
}

$pbxUser = $pbxInfo->findUserH323($faxJob->getUserH323());
if(!$pbxUser) {
    print getLogDate() . "User not found(" . $faxJob->getUserH323() . ").\n";

    $fax->handleIncomingCall($callHandle, "", "", "", true);
    
    $info = array();
    $info['<type>']             = 'RightsNoUser';
    $info['<sender>']           = $faxJob->getUserName();
    $info['<user-name>']        = $faxJob->getUserName();
    $info['<remote-number>']    = $faxJob->getRemoteNumber();

    handleError($faxDatabase, $faxJob, $cfg, $info);
}

$licensed   = null;

if($pbxUser->getLocation() != "" && $pbxUser->getLocation() != ".") {
    $locationPbxUrl = "";
    print getLogDate() . "Getting user location URL (" . $pbxUser->getLocation() . ")... ";
    try {
        $locationPbxUrl = $pbx->LocationUrl(true, true, true, true, $pbxUser->getLocation(), $https);
    }
    catch (Exception $e) {
    }
    if($locationPbxUrl != "") {
        print $locationPbxUrl . "\n";
        print getLogDate() . "Connecting to user location PBX...\n";
        $options['location'] = $locationPbxUrl;
        if($cfg->trace) {
            $options['trace'] = 1;
        }
        $locationPbx = new InnovaphonePBX(
            "",
            "",
            $https,
            $httpu,
            $httpp,
            "",
            "innovaphone PHP fax call in [" . $version . "]",
            $options,
            dirname(__FILE__) . '/../wsdl/pbx10_00.wsdl');

        $locationPbx->showVersion();
        print "\n";

        $locationPbxInfo = new InnovaphonePbxInfo($locationPbx);
            
        print getLogDate() . "Checking user rights... ";        
        $licensed        = $locationPbxInfo->userLicense($pbxUser, "Fax");
    }
}
else {
    print getLogDate() . "Checking user rights... ";
    $licensed   = $pbxInfo->userLicense($pbxUser, "Fax");
}

if(!$licensed) {
    if($licensed === false) {
        print "no license.\n";
    }
    else {
        print "failed.\n";
    }
    print getLogDate() . "Rejecting call... \n";
    $fax->handleIncomingCall($callHandle, "", "", "", true);

    $info = array();
    $info['<type>']             = $licensed === false ? 'RightsNoLicense' : 'RightsCheckFailed';
    $info['<sender>']           = $faxJob->getUserName();
    $info['<user-name>']        = $faxJob->getUserName();
    $info['<remote-number>']    = $faxJob->getRemoteNumber();
    handleError($faxDatabase, $faxJob, $cfg, $info);
}
print "ok.\n";


$destinationUserH323    = $destinationUser->getH323();
$m                      = $destinationUserH323;
$h323NamePart           = "";
for($i = 0; $i < strlen($m); $i++) {
    if(('a' <= $m[$i] && $m[$i] <= 'z') ||
        ('A' <= $m[$i] && $m[$i] <= 'Z') ||
        ('0' <= $m[$i] && $m[$i] <= '9') ||
        $m[$i] == '@')
    {
        $h323NamePart .= $m[$i];
    }
}                            
$fileName               = $configurationId . '-incoming-' . date('Ymd-His') . '-' . $faxJob->getGuid() . '.sff';
$faxJob->setFileName($webdavUrl . "/" . $fileName);

$faxJob = $faxDatabase->updateFaxJob($faxJob);
if(!$faxJob) {
    die("Could not update database object.");
}


print getLogDate() . "Accepting call... \n";
$subscriberId       = $cfg->faxSubscriberId;
$userNumberPrefix   = $cfg->faxUserNumberPrefix;
$restrictedNumber   = strtoupper(substr($destinationUserNumber, 0, 1)) == "R";

if($destinationUser->getPseudoType() == "map") {
    $subscriberId = $cfg->faxSubscriberIdAlt;
    if($subscriberId != "") {
        $subscriberId .= strpos($subscriberId, ' ') !== false ? " " : "";
        $subscriberId .= $destinationUserNumber;
    }
    $userNumberPrefix = "";
}
else if($subscriberId != "" && $cfg->faxAddUserNumber && $destinationUser != $user) {
    if(!strncmp($cfg->faxUserNumberPrefix, $destinationUserNumber, strlen($cfg->faxUserNumberPrefix))) {
        $subscriberId .= substr($destinationUserNumber, strlen($cfg->faxUserNumberPrefix));
    }
    else {
        $subscriberId .= $destinationUserNumber;
    }
}

if($restrictedNumber) {
    $subscriberId = "";
}

$fax->handleIncomingCall($callHandle, $faxJob->getFileName(), $subscriberId, $userNumberPrefix, false, $cfg->trace, $cfg->modemSpeed);

$page           = 0;
$connectedTime  = null;

do {
    $pbxInfo->purge();

    try {
        $p = $pbx->Poll($pbx->getSession());
    }
    catch(Exception $e) {
        try {
            $p = $pbx->Echo($pbx->getSession(), $pbx->getKey());
        }
        catch(Exception $e) {
            $p = 0;
            print getLogDate() . "Connection error: " . $e->faultstring . "\n";
        }
        
        if($p) {
            continue;
        }
        else {
            $pbx = 0;
            print getLogDate() . "Connection lost. Undefined state.\n\n";
            //incomplete, error handling
            break;
        }
    }

    $pbxInfo->update($p);
    $call = $fax->getCall($callHandle);
    if($call) {
        $call->setWebdavFileName($faxJob->getFileName());
        $faxShm->updateCall($call);
        if($call->getLastFaxState() != $call->getFaxState() ||
            $page != $call->getPage())
        {
            print getLogDate() . $call->getFaxState();
            if($call->getPage()) {
                print ", page " . $call->getPage();
            }
            $page = $call->getPage();
            if($call->getErrorCause()) {
                print ", " . $call->getErrorCause();
            }
            if($call->getErrorDescription()) {
                print ", " . $call->getErrorDescription();
            }
            print "...\n";
            $call->setLastState($call->getState());
            $call->setLastFaxState($call->getFaxState());
        }
        if(!$connectedTime && ($call->getState() & 0xF) == 5) {
            $connectedTime = new DateTime();
        }
    }

} while(!$call || $call->isActive());

print getLogDate() . "Call disconnected.\n";
$faxShm->updateJob($jobGuid, null, null);


if($call->getRemoteSubscriberId()) {
    $faxJob->setRemoteSubscriberId($call->getRemoteSubscriberId());
}

switch($call->getFaxState()) {
    case 'error':
    case 'aborted':
        $faxJob->setProgress('closed');
        $faxJob->setResult($call->getFaxState());
        if($call->getErrorCause())          $faxJob->setErrorCause($call->getErrorCause());
        if($call->getErrorDescription())    $faxJob->setErrorDescription($call->getErrorDescription());
        if($page > 1) {
            $faxJob->setProgress('queued');
            $faxJob->setPages($page - 1);
        }
        break;    

    case 'completed':
        $faxJob->setProgress('queued');
        $faxJob->setPages($call->getPages());
        break;    
    
    default:
        $faxJob->setProgress('closed');
        $faxJob->setResult('error');
        $faxJob->setErrorCause("script error");
        $faxJob->setErrorDescription("wrong fax call state");
        break;
}

if($connectedTime) {
    $now = new DateTime();
    $duration = $now->diff($connectedTime);
    $faxJob->setDuration($duration);
    $faxJob->setCallTimestamp($connectedTime);
}

print getLogDate() . "Update database... ";
$faxJob = $faxDatabase->updateFaxJob($faxJob);
if(!$faxJob || $faxJob->getProgress() == 'created') {
    print "failed.\n";
    die("Could not update database object.");
}
print "done.\n";


if($call->getFaxState() == 'error') {
    $info = array();
    $info['<type>']                 = 'Incoming';
    $info['<result>']               = $faxJob->getResult();
    $info['<user-name>']            = $faxJob->getUserName();
    $info['<user-h323>']            = $faxJob->getUserH323();
    $info['<user-number>']          = $faxJob->getUserNumber();
    $info['<remote-number>']        = $faxJob->getRemoteNumber();
    $info['<remote-subscriber-id>'] = $faxJob->getRemoteSubscriberId();
    $info['<calls>']                = $faxJob->getCalls();
    $info['<pages>']                = $faxJob->getPages();
    $info['<duration>']             = $faxJob->getDuration() ? $faxJob->getDuration()->format('%i:%S') : "-";
    $info['<error-cause>']          = $faxJob->getErrorCause();
    $info['<error-description>']    = $faxJob->getErrorDescription();
    $info['<date>']                 = $faxJob->getCallTimestamp() ? InputPageDateField::translateDate(($faxJob->getCallTimestamp()->getTimestamp())) : "-";
    $info['<time>']                 = $faxJob->getCallTimestamp() ? $faxJob->getCallTimestamp()->format("H:i:s") : "-";

    handleError($noDatabase, $faxJob, $cfg, $info);
}


$faxShm->unlockJob($jobGuid);

print getLogDate() . "Finished.\n";
?>
